package com.gic.haoban.manage.service.service.chat.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.JSONResponse;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.dubbo.entity.ProviderLocalTag;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.common.utils.DingUtils;
import com.gic.haoban.manage.api.dto.WxEnterpriseQwDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatPlanDTO;
import com.gic.haoban.manage.api.dto.chat.GroupChatPlanLogDTO;
import com.gic.haoban.manage.api.dto.qdto.chat.GroupChatPlanSearchQDTO;
import com.gic.haoban.manage.api.enums.PendingTaskTypeEnum;
import com.gic.haoban.manage.api.enums.chat.PlanSendStatusEnum;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.dao.mapper.StaffMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatPlanLogMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatPlanMapper;
import com.gic.haoban.manage.service.dao.mapper.chat.GroupChatPlanOwnerLogMapper;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatPlan;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatPlanLog;
import com.gic.haoban.manage.service.entity.chat.TabGroupChatPlanOwnerLog;
import com.gic.haoban.manage.service.pojo.bo.PendingTaskBO;
import com.gic.haoban.manage.service.pojo.bo.chat.GroupChatBO;
import com.gic.haoban.manage.service.pojo.bo.chat.GroupChatPlanBO;
import com.gic.haoban.manage.service.service.QywxSendService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.service.chat.GroupChatPlanOwnerLogService;
import com.gic.haoban.manage.service.service.chat.GroupChatPlanService;
import com.gic.haoban.manage.service.service.chat.GroupChatService;
import com.gic.haoban.manage.service.service.notify.PendingTaskService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.quartz.api.dto.QuartzTaskDTO;
import com.gic.quartz.api.service.QuartzService;
import com.gic.wechat.api.dto.qywx.QywxSendMessageResultDTO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.github.pagehelper.PageHelper;

@Service("groupChatPlanService")
public class GroupChatPlanServiceImpl implements GroupChatPlanService {

    private static org.apache.logging.log4j.Logger logger = LogManager.getLogger(GroupChatPlanService.class);

    @Autowired
    private GroupChatPlanMapper groupChatPlanMapper;
    @Autowired
    private GroupChatPlanOwnerLogService groupChatPlanOwnerLogService;
    @Autowired
    private GroupChatPlanOwnerLogMapper groupChatPlanOwnerLogMapper;
    @Autowired
    private GroupChatPlanLogMapper groupChatPlanLogMapper;
    @Autowired
    private QywxSendService qywxSendService;
    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private Config config;
    @Autowired
    private QywxSuiteApiService qywxSuiteApiService;
    @Autowired
    private StaffMapper staffMapper;
    @Autowired
    private GroupChatService groupChatService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private QuartzService quartzService;
    @Autowired
    private PendingTaskService pendingTaskService;

    private static GicMQClient mqClient = GICMQClientUtil.getClientInstance();

    // 每次成员数
    private int pageSize = 1000;

    // 发送队列
    private void putMQForSend(Long id) {
        try {
            mqClient.sendCommonMessage("haobanChatMQ", id + "",
                    "com.gic.haoban.manage.api.service.chat.GroupChatPlanApiService", "doPlanTimer");
        } catch (Exception e) {
            logger.error("发送MQ异常");
            e.printStackTrace();
        }
    }

    // 统计队列
    private void putMQForData(Long id) {
        try {
            mqClient.sendCommonMessage("haobanChatMQ", id + "",
                    "com.gic.haoban.manage.api.service.chat.GroupChatPlanApiService", "doPlanSendInfoTimer");
        } catch (Exception e) {
            logger.error("发送MQ异常");
            e.printStackTrace();
        }
    }

    @Override
    public void save(GroupChatPlanDTO dto) {
        Long planId = dto.getPlanId();
        TabGroupChatPlan entity = EntityUtil.changeEntityByJSON(TabGroupChatPlan.class, dto);
        if (entity.getSendType() == 1) {
            entity.setSendTime(new Date());
        }
        entity.setStartTime(entity.getSendTime());
        Date endTime = DateUtil.addDay(entity.getStartTime(), entity.getExpireDays());
        entity.setEndTime(DateUtil.getEndTimeOfDay(endTime));
        String staffIdList = dto.getStaffIdList();
        if (null == planId) {
            planId = UniqueIdUtils.uniqueLong();
            entity.setPlanId(planId);
            entity.setStatusFlag(1);
            entity.setStaffCount(staffIdList.split(",").length);
            entity.setSendCount(0);
            entity.setSuccessChatCount(0);
            entity.setFailChatCount(0);
            this.groupChatPlanMapper.insert(entity);
            // 新增群主日志
            this.groupChatPlanOwnerLogService.batchAdd(dto.getStaffIdList(), entity);
        } else {
            TabGroupChatPlan old = this.groupChatPlanMapper.selectById(planId) ;
            int staffCount = old.getStaffCount();
            if (StringUtils.isNotBlank(dto.getDelStaffIdList())) {
                staffCount = staffCount - dto.getDelStaffIdList().split(",").length;
            }
            if (StringUtils.isNotBlank(dto.getStaffIdList())) {
                staffCount = staffCount + dto.getStaffIdList().split(",").length;
            }
            entity.setStaffCount(staffCount);
            this.groupChatPlanMapper.updateById(entity);
            // 删除本次删除的
            this.groupChatPlanOwnerLogService.deleteByStaffIds(planId, dto.getDelStaffIdList());
            // 新增本次新增的
            this.groupChatPlanOwnerLogService.batchAdd(staffIdList, entity);
        }
        // 立即发送
        if (entity.getSendType() == 1) {
            this.addOrDelTimer(planId, entity.getSendTime(), 0);
            this.putMQForSend(planId);
        } else {
            this.addOrDelTimer(planId, entity.getSendTime(), 1);
        }
    }

    @Override
    public GroupChatPlanBO getById(Long planId) {
        TabGroupChatPlan plan = this.groupChatPlanMapper.selectById(planId);
        GroupChatPlanBO bo = EntityUtil.changeEntityByJSON(GroupChatPlanBO.class, plan);
        return bo;
    }

    @Override
    public Page<GroupChatPlanBO> listPage(GroupChatPlanSearchQDTO qdto, BasePageInfo basePageInfo) {
        if (StringUtils.isNotBlank(qdto.getSearchParams())) {
            String params = qdto.getSearchParams();
            qdto.setSearchParams("%" + params + "%");
        }
        PageHelper.startPage(basePageInfo);
        List<TabGroupChatPlan> list = this.groupChatPlanMapper.list(qdto);
        Page<GroupChatPlanBO> retPage = PageHelperUtils.changePageHelperToCurrentPage(list, GroupChatPlanBO.class);
        return retPage;
    }

    @Override
    public void doPlan(Long planId) {
        logger.info("开始群群发发送{}", planId);
        DingUtils.send("开始群群发发送,planId=" + planId + ",tranceid=" + ProviderLocalTag.tag.get().traceId, false);
        TabGroupChatPlan plan = this.groupChatPlanMapper.selectById(planId);
        if (null == plan) {
            logger.info("计划不存在,planid={}", planId);
            return;
        }
        if (plan.getSendFlag() == PlanSendStatusEnum.SENDED.getCode()) {
            logger.info("计划已执行过,planid={}", planId);
            return;
        }
        this.groupChatPlanMapper.updateSendFlag(planId);
        String wxEnterpriseId = plan.getWxEnterpriseId();
        String chatContent = plan.getChatContent();
        JSONArray jsonArr = JSONArray.parseArray(chatContent);
        List<String> sendIdList = new ArrayList<>();
        for (int i = 0; i < jsonArr.size(); i++) {
            JSONObject json = jsonArr.getJSONObject(i);
            sendIdList.add(json.getString("relation_id"));
        }
        int pageNum = 0;
        List<TabGroupChatPlanOwnerLog> ownerList = null;
        while (true) {
            ownerList = this.groupChatPlanOwnerLogMapper.listForDoPlan(planId, pageNum * pageSize, pageSize);
            if (CollectionUtils.isEmpty(ownerList)) {
                break;
            }
            pageNum++;
            for (TabGroupChatPlanOwnerLog ownerLog : ownerList) {
                try {
                    String staffId = ownerLog.getStaffId();
                    ServiceResponse<String> sendResp = qywxSendService.sendChatMessage(wxEnterpriseId, staffId,
                            sendIdList);
                    if (sendResp.isSuccess()) {
                        String msgid = sendResp.getResult();
                        ownerLog.setMsgid(msgid);
                        ownerLog.setSendStatus(PlanSendStatusEnum.TO_BE_SEND.getCode());
                    } else {
                        ownerLog.setRemark(sendResp.getMessage().substring(0, 500));
                        ownerLog.setSendStatus(PlanSendStatusEnum.FAIL.getCode());
                    }
                    ownerLog.setWxEnterpriseId(wxEnterpriseId);
                    this.groupChatPlanOwnerLogService.update(ownerLog);
                    // 代办
                    if (ownerLog.getSendStatus() == PlanSendStatusEnum.TO_BE_SEND.getCode()
                            && StringUtils.isNotBlank(ownerLog.getClerkId())
                            && StringUtils.isNotBlank(ownerLog.getMsgid())) {
                        this.savePendTask(ownerLog, plan);
                    }
                } catch (Exception e) {
                    logger.info("群群发任务异常,ownerLog={}", ownerLog.getOwnerLogId());
                }
            }
        }
    }

    private void savePendTask(TabGroupChatPlanOwnerLog ownerLog, TabGroupChatPlan plan) {
        logger.info("创建导购待办={}", JSON.toJSONString(ownerLog));
        PendingTaskBO task = new PendingTaskBO();
        task.setRelationId(ownerLog.getOwnerLogId() + "");
        task.setBusinessId(ownerLog.getPlanId() + "");
        task.setTaskType(PendingTaskTypeEnum.GROUP_CHAT.getType());
        task.setTitle(plan.getName());
        task.setInvalidTime(plan.getEndTime());
        task.setDescription(plan.getRemark());
        task.setStoreId(ownerLog.getStoreId());
        task.setClerkId(ownerLog.getClerkId());
        task.setEnterpriseId(ownerLog.getEnterpriseId());
        task.setOverdueTime(plan.getEndTime());
        this.pendingTaskService.addOrUpdatePendingTask(task);
    }

    @Override
    public void doPlanSendInfoTimer(String params) {
        DingUtils.send("开始群群发统计(定时),params=" + params + ",tranceid=" + ProviderLocalTag.tag.get().traceId, false);
        Long planId = Long.parseLong(params);
        List<Long> idList = this.groupChatPlanOwnerLogService.listForTimer(planId);
        logger.info("定时处理群发结果,条数={}", idList.size());
        for (Long ownerLogId : idList) {
            try {
                this.doPlanSendInfo(ownerLogId);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    public void doPlanSendInfo(Long ownerLogId) {
        logger.info("开始群群发统计{}", ownerLogId);
        TabGroupChatPlanOwnerLog ownerLog = this.groupChatPlanOwnerLogService.getById(ownerLogId);
        if (ownerLog.getSendStatus() == 2) {
            logger.info("已统计过");
            return;
        }
        logger.info("开始群群发统计{}", ownerLog.getMsgid());
        Long planId = ownerLog.getPlanId();
        TabGroupChatPlan plan = this.groupChatPlanMapper.selectById(planId);
        if (null == plan) {
            logger.info("计划不存在{}", planId);
            return;
        }
        String wxEnterpriseId = ownerLog.getWxEnterpriseId();
        String enterpriseId = ownerLog.getEnterpriseId();
        String msgid = ownerLog.getMsgid();
        String staffId = ownerLog.getStaffId();
        WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId);
        TabHaobanStaff staff = this.staffMapper.selectByPrimaryKey(staffId);
        if (null == staff) {
            logger.info("成员不存在{}", staffId);
            return;
        }
        String userid = staff.getWxUserId();
        if (qwDTO.needOpenUserId3th()) {
            userid = staff.getWxOpenUseId();
        }
        JSONResponse respon = qywxSuiteApiService.resultExternalMessage(qwDTO.getThirdCorpid(), config.getWxSuiteid(),
                msgid, null, userid);
        logger.info("统计企微返回={}", JSON.toJSONString(respon));
        if (respon.getErrorCode() != 0) {
            return;
        }
        List<QywxSendMessageResultDTO> list = (List<QywxSendMessageResultDTO>) respon.getList();
        if (CollectionUtils.isEmpty(list)) {
            logger.info("企微返回列表空");
            return;
        }
        int sendCount = 0;
        int failCount = 0;
        List<TabGroupChatPlanLog> addList = new ArrayList<>();
        boolean endFlag = false ;
        for (QywxSendMessageResultDTO item : list) {
            TabGroupChatPlanLog entity = new TabGroupChatPlanLog();
            Date sendTime = item.getSendTime();
            String wxChatId = item.getChatId();
            // 如果企微的发送时间晚于计划的结束时间,不记录此类数据
            if (sendTime.after(plan.getEndTime())) {
                logger.info("发送时间晚计划结束");
                endFlag = true ;
                continue;
            }
            GroupChatBO chat = this.groupChatService.getByWxChatId(wxEnterpriseId, wxChatId);
            if (null == chat || StringUtils.isBlank(chat.getEnterpriseId())) {
                logger.info("群不存在,或未认领,wxChatId={}", wxChatId);
                continue;
            }
            if (!chat.getEnterpriseId().equals(plan.getEnterpriseId())) {
                logger.info("群的商户id和计划的商户id不一致{},{}", chat.getEnterpriseId(), plan.getEnterpriseId());
                continue;
            }
            entity.setGroupChatId(chat.getGroupChatId());
            // 企微返回成功
            // 0-未发送 1-已发送 2-因客户不是好友导致发送失败 3-因客户已经收到其他群发消息导致发送失败
            if (item.getStatus() == 1) {
                sendCount++;
                // 已发送
                entity.setSendStatus(PlanSendStatusEnum.SENDED.getCode());
            } else {
                failCount++;
                // 发送失败
                entity.setSendStatus(PlanSendStatusEnum.FAIL.getCode());
            }
            entity.setLogId(UniqueIdUtils.uniqueLong());
            entity.setPlanId(ownerLog.getPlanId());
            entity.setOwnLogId(ownerLog.getOwnerLogId());
            entity.setWxEnterpriseId(wxEnterpriseId);
            entity.setEnterpriseId(enterpriseId);
            entity.setStaffId(staffId);
            entity.setSendTime(sendTime);
            addList.add(entity);
        }
        if (CollectionUtils.isNotEmpty(addList)) {
            this.groupChatPlanLogMapper.batchInsert(addList);
        }
        // 更新统计数量、状态
        if(!endFlag) {
            this.groupChatPlanOwnerLogService.updateSendCount(ownerLog.getOwnerLogId(), sendCount,
                    list.get(0).getSendTime(), failCount);
            // 完成代办
            this.pendingTaskService.updateFinish(ownerLog.getOwnerLogId() + "");
            // 更新计划统计
            this.groupChatPlanMapper.updateData(ownerLog.getPlanId(), 1, sendCount, failCount);
        }
    }

    @Override
    public void del(String clerkId, String clerkName, Long planId) {
        this.addOrDelTimer(planId, new Date(), 0);
        this.groupChatPlanMapper.deleteById(planId, clerkId, clerkName);
    }

    @Override
    public void stop(String clerkId, String clerkName, Long planId) {
        logger.info("终止计划,并触发统计planId={}", planId);
        Integer modifyStartTimeFlag = null;
        TabGroupChatPlan plan = this.groupChatPlanMapper.selectById(planId);
        if (plan.getStartTime().getTime() >= new Date().getTime()) {
            modifyStartTimeFlag = 1;
        }
        this.groupChatPlanMapper.stopById(planId, clerkId, clerkName, modifyStartTimeFlag);
        this.putMQForData(planId);
    }

    @Override
    public Page<GroupChatPlanLogDTO> listLogPage(GroupChatPlanSearchQDTO qdto, BasePageInfo basePageInfo) {
        PageHelper.startPage(basePageInfo);
        if (StringUtils.isNotBlank(qdto.getSearchParams())) {
            String params = qdto.getSearchParams();
            qdto.setSearchParams("%" + params + "%");
        }
        if (StringUtils.isNotBlank(qdto.getChatName())) {
            String params = qdto.getChatName();
            qdto.setChatName("%" + params + "%");
            qdto.setWxChatId(params+"%");
        }
        List<GroupChatPlanLogDTO> list = this.groupChatPlanLogMapper.listPlanLog(qdto);
        if (CollectionUtils.isNotEmpty(list)) {
            Set<String> clerkIdList = list.stream().filter(dto -> StringUtils.isNotBlank(dto.getClerkId()))
                    .map(dto -> dto.getClerkId()).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(clerkIdList)) {
                List<ClerkDTO> clerkList = this.clerkService.getClerkByClerkIds(clerkIdList);
                if (CollectionUtils.isNotEmpty(clerkList)) {
                    Map<String, ClerkDTO> map = clerkList.stream()
                            .collect(Collectors.toMap(ClerkDTO::getClerkId, dto -> dto, (k1, k2) -> k1));
                    list.forEach(dto -> {
                        String clerkId = dto.getClerkId();
                        if (StringUtils.isNotBlank(clerkId) && null != map.get(clerkId)) {
                            dto.setClerkCode(map.get(clerkId).getClerkCode());
                            dto.setClerkName(map.get(clerkId).getClerkName());
                        }
                    });
                }
            }
            List<String> storeIdList = list.stream().filter(dto -> StringUtils.isNotBlank(dto.getStoreId()))
                    .map(dto -> dto.getStoreId()).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(storeIdList)) {
                String[] arr = new String[storeIdList.size()];
                for (int i = 0; i < storeIdList.size(); i++) {
                    arr[i] = storeIdList.get(i);
                }
                List<StoreDTO> storeList = this.storeService.getStores(arr);
                Map<String, StoreDTO> map = storeList.stream()
                        .collect(Collectors.toMap(StoreDTO::getStoreId, dto -> dto, (k1, k2) -> k1));
                list.forEach(dto -> {
                    String storeId = dto.getStoreId();
                    if (StringUtils.isNotBlank(storeId) && null != map.get(storeId)) {
                        dto.setStoreCode(map.get(storeId).getStoreCode());
                        dto.setStoreName(map.get(storeId).getStoreName());
                    }
                });
            }
        }
        Page<GroupChatPlanLogDTO> retPage = PageHelperUtils.changePageHelperToCurrentPage(list,
                GroupChatPlanLogDTO.class);
        return retPage;
    }

    public void addOrDelTimer(Long planId, Date sendTime, int status) {
        logger.info("添加1/删除0定时,{},{}", status, planId);
        QuartzTaskDTO quartzTask = new QuartzTaskDTO();
        quartzTask.setTaskStatus(status);
        String taskTimeInfo = this.getTaskTime(sendTime);
        quartzTask.setTaskTimeInfo(taskTimeInfo);
        quartzTask.setTaskService("com.gic.haoban.manage.api.service.chat.GroupChatPlanApiService");
        quartzTask.setTaskMethod("doPlanTimer");
        quartzTask.setTaskParam(planId + "");
        quartzTask.setReferId(planId + "");
        try {
            quartzService.addOrUpdateQuartzTask(quartzTask);
        } catch (Exception e) {
            logger.warn("提交定时任务异常", e);
        }
    }

    private String getTaskTime(Date date) {
        String timeInfo = DateUtil.dateToStr(date, "yyyyMMddHHmmss");
        String year = timeInfo.substring(0, 4);
        int month = Integer.parseInt(timeInfo.substring(4, 6));
        int day = Integer.parseInt(timeInfo.substring(6, 8));
        int hour = Integer.parseInt(timeInfo.substring(8, 10));
        int minute = Integer.parseInt(timeInfo.substring(10, 12));
        int second = Integer.parseInt(timeInfo.substring(12, 14));
        String time = new StringBuilder().append(second).append(" ").append(minute).append(" ").append(hour).append(" ")
                .append(day).append(" ").append(month).append(" ").append(year).toString();
        return time;
    }

    @Override
    public void updateRemark(GroupChatPlanDTO dto) {
        this.groupChatPlanMapper.updateRemark(dto);
    }

}